

namespace GossamerVR.Stripwise
{
    using System;    
    using UnityEngine;
    using GPUTools.Cloth.Scripts;

    /// <summary>
    /// A data container representing a clothing item that Stripwise is tracking.
    /// </summary>
    public class ClothingEntry
    {        
        public ClothSettings Settings;
        public DAZClothingItem Clothing;
        public bool Ignore;
        public bool IsSimulated => this.Clothing.type == DAZDynamicItem.Type.Sim || this.Clothing.type == DAZDynamicItem.Type.Custom;
        public bool IsReady => this.Settings != null;
        public bool IsActive => this.Clothing.active;
        public bool IsSimActive => this.IsReady && this.IsActive && this.Settings.enabled;        
        public bool IsValid => this.IsReady && this.Settings.Runtime?.Particles != null && this.Settings.Runtime.PointJoints != null;
        public long InitialLoadTime = 0;

        public ClothingEntry(DAZClothingItem clothing)
        {
            this.Clothing = clothing;
            this.Settings = clothing.transform.GetComponentInChildren<ClothSettings>(true);
            
        }

        /// <summary>
        /// Attempts to load simulation settings. The constructor may not have caught them,
        /// because the entry may have been created while the clothes are still loading.
        /// the clothing remains a pottentially valid candidate for settings, even if they're not found yet.
        /// Thus, this method checks to see if they are available.
        /// returns false if we can't find clothing settings within a minute of trying.
        /// </summary>
        /// <returns></returns>
        public bool LoadSettings()
        {
            this.Settings = this.Clothing?.transform?.GetComponentInChildren<ClothSettings>();
            long now = 0;
            if (this.Settings == null)
            {
                // we don't want to keep trying to load forever.
                // if for some reason the clothing hasn't loaded a simulation setting in 10 seconds,
                // or if somehow it remains in a pending state after the settings object is destroyed, 
                // then we're going to stop looking.
                now = DateTimeOffset.Now.Ticks / TimeSpan.TicksPerMillisecond;
                if (this.InitialLoadTime == 0)
                {
                    this.InitialLoadTime = now;
                }
            }
            else this.InitialLoadTime = 0;
                       

            // we return false if we havent found it and it's been over 60 seconds.
            return this.Settings != null || (now - this.InitialLoadTime <= 60000);
        }
    }
}